<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +--------------------------------------------------------+
// | PHP version 5.0.x                                      |
// +--------------------------------------------------------+
// | Copyright (c) 2005 Song Hyo-Jin                        |
// +--------------------------------------------------------+
// | License : GPL (current new version)                    |
// +--------------------------------------------------------+
// | Author : Song Hyo-Jin <crosser at hanmail dot net>     |
// |                                  (MSN Messengerable)   |
// +--------------------------------------------------------+
//
// $Id: SHJMSSQL.inc, v 0.1.1 2005/08/11 19:40:00 crosser Exp $
//
// MS SQL database control

include_once 'SHJ/debug.inc';
include_once 'SHJ/DB/CommonSQL.inc';

class SHJMSSQL
{
	public $result, $err_report, $query;
	private $transaction, $query_count;
	protected $persistent, $conn, $db_name;

	public function __construct($db_ini_file, $persistent = false)
	{
		global $_DEBUG;
		$this->err_report = $_DEBUG;

		$this->transaction	= false;
		$this->query_count	= 0;
		$this->persistent = $persistent;
		static $db_set_file = array();

		if(false === array_key_exists($db_ini_file, $db_set_file)) {
			if(!($db_set_file[$db_ini_file] = parse_ini_file($db_ini_file))) {
				exit($db_ini_file.' not found');
			}
		}
		$myset = &$db_set_file[$db_ini_file];
		if($this->persistent) {
			$this->conn = mssql_pconnect($myset['dsn'], $myset['user'], $myset['pass']);
		} else {
			$this->conn = mssql_connect($myset['dsn'], $myset['user'], $myset['pass']);
		}

		if(isset($myset['db'])) {
			$this->db_name = $myset['db'];
			mssql_select_db($this->db_name, $this->conn);
		} else {
			$this->db_name = '';
		}

		return true;
	}

	public function begin()
	{
		if(!$this->transaction) {
			mssql_query('BEGIN TRAN', $this->conn);
		} else {
			throw new Exception('already begined.');
			return false;
		}

		$this->transaction = true;

		return true;
	}

	public function commit()
	{
		if($this->transaction) {
			mssql_query('COMMIT TRAN', $this->conn);
		} else {
			throw new Exception('not yet begin.');
			return false;
		}

		$this->transaction = false;

		return true;
	}

	public function rollback()
	{
		if($this->transaction) {
			mssql_query('ROLLBACK TRAN', $this->conn);
		} else {
			throw new Exception('not yet begin.');
			return false;
		}

		$this->transaction = false;

		return true;
	}

	public function __destruct()
	{
		if($this->conn !== false) {
			$this->close();
		}

		return true;
	}

	public function close()
	{
		if($this->conn === false) {
			return false;
		}
		if($this->transaction) {
			$this->rollback();
		}
		if($this->persistent === false) {
			mssql_close($this->conn);
		}
		$this->conn = false;

		return true;
	}

	public function query($query)
	{
		$this->query = $query;
		$this->exec();

		return true;
	}

	public function exec()
	{
		$this->query_count ++;

		try {
			$this->result = mssql_query($this->query, $this->conn);
		} catch(Exception $e) {
			$this->result = false;
		}
		if(!$this->result) {
			if($this->err_report) {
				throw new Exception('<p>query <strong>'.htmlspecialchars($this->query).'</strong> error (count : '.$this->query_count.' )</p><xmp>'.$e->getMessage().'</xmp>');
			} else {
				throw new Exception('<p>query error (count : '.$this->query_count.' )</p>');
			}
		}
		return true;
	}

	public function rows()
	{
		return mssql_num_rows($this->result);
	}

	public function id()
	{
		$this->query('SELECT @@IDENTITY');
		return $this->field();
	}

	public function affected_rows()
	{
		return mssql_rows_affected($this->conn);
	}

	public function field($row = 0, $col = 0)
	{
		return mssql_result($this->result, $row, $col);
	}

	public function fetch()
	{
		return mssql_fetch_object($this->result);
	}

	public function fetch_row()
	{
		return mssql_fetch_row($this->result);
	}

	public function fetch_assoc()
	{
		return mssql_fetch_assoc($this->result);
	}

	public function fetch_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function fetch_row_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch_row())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function fetch_assoc_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch_assoc())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function select($tables, $fields, $wheres = '', $orders = '', $offset = 0, $limit = -1, $pkey = false)
	{
		if(is_array($tables)) {
			$table_name = implode(', ', $tables);
		} else {
			$table_name = $tables;
		}
		if(is_array($fields)) {
			$field_name = implode(', ', $fields);
		} else {
			$field_name = $fields;
		}
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == 1 or $where == '()') {
			$where = '';
		} elseif($where != '') {
			$where = ' WHERE '.$where;
		}
		if(is_array($orders)) {
			$order		= implode(', ', $orders);
		} else {
			$order		= $orders;
		}
		if($order == 1) {
			$order		= '';
		} elseif($order != '') {
			$order		= ' ORDER BY '.$order;
		}

		if($limit != null) {
			if($limit == -1) {
				$limit	= '';
			} elseif(!preg_match('/^\d+$/', $limit)) {
				throw new Exception('limit must numeric');
				return false;
			}
		} else {
			$limit = '';
		}

		if($offset != null) {
			if(!preg_match('/^\d+$/', $offset)) {
				throw new Exception('offset must numeric');
				return false;
			}
			if($offset == 0) {
				$offset	= '';
			}
		} else {
			$offset = '';
		}

		if($limit == '' && $offset == '') {
			$this->query('SELECT '.$field_name.' FROM '.$table_name.$where.$order);
		} elseif($limit != '' && $offset == '') {
			$this->query('SELECT TOP '.$limit.' '.$field_name.' FROM '.$table_name.$where.$order);
		} elseif($limit == '' && $offset != '') {
			if(!$pkey) {
				$pkey = $this->get_pkey($table_name);
			}
			$this->query('SELECT '.$field_name.' FROM '.$table_name.' WHERE '.$pkey.' NOT IN (SELECT TOP '.($offset + 1).' '.$pkey.' FROM '.$table_name.$where.$order.')'.$order);
		} else {
			if(!$pkey) {
				$pkey = $this->get_pkey($table_name);
			}
			if($order != '') {
				if(is_array($orders)) {
					foreach($orders as $value) {
						if($value == '1') {
							continue;
						}
						$rev_order[] = preg_replace_callback('/^[^\s]+(\s+(ASC|DESC))?$/i', 'order_rev_callback', $value);
					}
				} else {
					$rev_order[] = preg_replace_callback('/^([^\s]+)(\s+(ASC|DESC))?$/i', 'order_rev_callback', $orders);
				}
				if(!isset($rev_order)) {
					$order = ' ORDER BY 1';
					$rorder = ' ORDER BY 1 DESC';
				} else {
					$rorder = ' ORDER BY '.implode(', ', $rev_order);
				}
			} else {
				$order = ' ORDER BY 1';
				$rorder = ' ORDER BY 1 DESC';
			}
			$this->query = 'SELECT '.$field_name.' FROM '.$table_name.' WHERE '.$pkey.' IN (SELECT TOP '.$limit.' '.$pkey.' FROM '.$table_name.' WHERE '.$pkey.' IN (SELECT TOP '.($offset + $limit).' '.$pkey.' FROM '.$table_name.$where.$order.')'.$rorder.')'.$order;
		}

		return true;
	}

	public function insert($table_name, $fields)
	{
		foreach($fields as $key => $value) {
			$fieldname[] = $key;
			if(substr($value, 0, 1) == CHR00) {
				$inserts[] = substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$inserts[] = $value;
			} else {
				$inserts[] = '\''.$value.'\'';
			}
		}

		$this->query('INSERT INTO '.$table_name.' ('.implode(', ', $fieldname).') VALUES ('.implode(', ', $inserts).')');

		return true;
	}

	public function update($table_name, $fields, $wheres = 1)
	{
		foreach($fields as $key => $value) {
			if(substr($value, 0, 1) == CHR00) {
			$updates[] = $key.'='.substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$updates[] = $key.'='.$value;
			} else {
				$updates[] = $key.'=\''.$value.'\'';
			}
		}
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == '' || $where == '()') {
			$where = 1;
		}

		$this->query('UPDATE '.$table_name.' SET '.implode(', ', $updates).' WHERE '.$where);

		return true;
	}

	/*public function replace($table_name, $fields)
	{
		foreach($fields as $key => $value) {
			$fieldname[] = $key;
			if(substr($value, 0, 1) == CHR00) {
				$replaces[] = substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/s', $value)) {
				$replaces[] = $value;
			} else {
				$replaces[] = '\''.$value.'\'';
			}
		}

		$this->query('REPLACE INTO '.$table_name.' ('.implode(', ', $fieldname).') VALUES ('.implode(', ', $replaces).')');

		return true;
	}*/

	public function delete($table_name, $wheres = '')
	{
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == '' || $where == '()') {
			return false;
		}

		$this->query('DELETE FROM '.$table_name.' WHERE '.$where);

		return true;
	}

	public function truncate($table_name)
	{
		$this->query('TRUNCATE TABLE '.$table_name);

		return true;
	}

	private function get_pkey($table_name)
	{
		$this->query('SELECT COLUMN_NAME FROM Information_Schema.Key_Column_Usage WHERE TABLE_NAME = \''.$table_name.'\' AND CONSTRAINT_NAME LIKE \'PK%\'');
		if(false === ($pkey = $this->field())) {
			throw new Exception($table_name.' table has no primary key.');
			return false;
		}

		return $pkey;
	}
}

function order_rev_callback($matches)
{
	if(isset($matches[3])) {
		if(strtoupper($matches[3]) == 'DESC') {
			return $matches[1].' ASC';
		}
	}
	return $matches[1].' DESC';
}

?>
