<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +--------------------------------------------------------+
// | PHP version 5.0.x                                      |
// +--------------------------------------------------------+
// | Copyright (c) 2005 Song Hyo-Jin                        |
// +--------------------------------------------------------+
// | License : GPL (current new version)                    |
// +--------------------------------------------------------+
// | Author : Song Hyo-Jin <crosser at hanmail dot net>     |
// |                                  (MSN Messengerable)   |
// +--------------------------------------------------------+
//
// $Id: SHJMySQL.inc, v 0.1.1 2005/08/11 19:40:00 crosser Exp $
//
// MySQL 4.1.x database control

include_once 'SHJ/debug.inc';
include_once 'SHJ/DB/CommonSQL.inc';

class SHJMySQL
{
	public $result, $err_report, $query;
	private $transaction, $query_count;
	protected $persistent, $conn, $db_name;

	public function __construct($db_ini_file, $persistent = false, $charset = false)
	{
		global $_DEBUG;
		$this->err_report = $_DEBUG;

		$this->transaction	= false;
		$this->query_count	= 0;
		$this->persistent = $persistent;
		static $db_set_file = array();

		if(false === array_key_exists($db_ini_file, $db_set_file)) {
			if(!($db_set_file[$db_ini_file] = parse_ini_file($db_ini_file))) {
				exit($db_ini_file.' not found');
			}
		}
		$myset = &$db_set_file[$db_ini_file];
		if($this->persistent) {
			$this->conn = mysql_pconnect($myset['host'], $myset['user'], $myset['pass']);
		} else {
			$this->conn = mysql_connect($myset['host'], $myset['user'], $myset['pass'], true);
		}

		$this->db_name = $myset['db'];
		mysql_select_db($this->db_name, $this->conn);

		if($charset) {
			// Other MySQL values are all utf8
			$this->query('SET NAMES '.$charset);
		}

		return true;
	}

	public function begin()
	{
		if(!$this->transaction) {
			mysql_query('BEGIN', $this->conn);
		} else {
			throw new Exception('already begined.');
			return false;
		}

		$this->transaction = true;

		return true;
	}

	public function commit()
	{
		if($this->transaction) {
			mysql_query('COMMIT', $this->conn);
		} else {
			throw new Exception('not yet begin.');
			return false;
		}

		$this->transaction = false;

		return true;
	}

	public function rollback()
	{
		if($this->transaction) {
			mysql_query('ROLLBACK', $this->conn);
		} else {
			throw new Exception('not yet begin.');
			return false;
		}

		$this->transaction = false;

		return true;
	}

	public function __destruct()
	{
		if($this->conn !== false) {
			$this->close();
		}

		return true;
	}

	public function close()
	{
		if($this->conn === false) {
			return false;
		}
		if($this->transaction) {
			$this->rollback();
		}
		if($this->persistent === false) {
			mysql_close($this->conn);
		}
		$this->conn = false;

		return true;
	}

	public function query($query)
	{
		$this->query = $query;
		$this->exec();

		return true;
	}

	public function exec()
	{
		$this->query_count ++;

		try {
			$this->result = mysql_query($this->query, $this->conn);
		} catch(Exception $e) {
			$this->result = false;
		}
		if(!$this->result) {
			if($this->err_report) {
				throw new Exception('<p>query <strong>'.htmlspecialchars($this->query).'</strong> error (count : '.$this->query_count.' )</p><xmp>'.mysql_error($this->conn).'</xmp>');
			} else {
				throw new Exception('<p>query error (count : '.$this->query_count.' )</p>');
			}
		}
		return true;
	}

	public function rows()
	{
		return mysql_num_rows($this->result);
	}

	public function id()
	{
		return mysql_insert_id($this->conn);
	}

	public function affected_rows()
	{
		return mysql_affected_rows($this->conn);
	}

	public function field($row = 0, $col = 0)
	{
		return mysql_result($this->result, $row, $col);
	}

	public function fetch()
	{
		return mysql_fetch_object($this->result);
	}

	public function fetch_row()
	{
		return mysql_fetch_row($this->result);
	}

	public function fetch_assoc()
	{
		return mysql_fetch_assoc($this->result);
	}

	public function fetch_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function fetch_row_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch_row())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function fetch_assoc_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch_assoc())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function select($tables, $fields, $wheres = '', $orders = '', $offset = 0, $limit = -1)
	{
		if(is_array($tables)) {
			$table_name = implode(', ', $tables);
		} else {
			$table_name = $tables;
		}
		if(is_array($fields)) {
			$field_name = implode(', ', $fields);
		} else {
			$field_name = $fields;
		}
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == 1 or $where == '()') {
			$where = '';
		} elseif($where != '') {
			$where = ' WHERE '.$where;
		}
		if(is_array($orders)) {
			$order		= implode(', ', $orders);
		} else {
			$order		= $orders;
		}
		if($order == 1) {
			$order		= '';
		} elseif($order != '') {
			$order		= ' ORDER BY '.$order;
		}

		if($limit != null) {
			if($limit == -1) {
				$limit	= '';
			} elseif(!preg_match('/^\d+$/', $limit)) {
				throw new Exception('limit must numeric');
				return false;
			} else {
				$limit	= ' LIMIT '.$limit;
			}
		} else {
			$limit = '';
		}

		if($offset != null) {
			if(!preg_match('/^\d+$/', $offset)) {
				throw new Exception('offset must numeric');
				return false;
			}
			if($offset == 0) {
				$offset	= '';
			} else {
				$offset	= ' OFFSET '.$offset;
			}
		} else {
			$offset = '';
		}

		$this->query = 'SELECT '.$field_name.' FROM '.$table_name.$where.$order.$limit.$offset;

		return true;
	}

	public function insert($table_name, $fields)
	{
		foreach($fields as $key => $value) {
			$fieldname[] = $key;
			if(substr($value, 0, 1) == CHR00) {
				$inserts[] = substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$inserts[] = $value;
			} else {
				$inserts[] = '\''.$value.'\'';
			}
		}

		$this->query('INSERT INTO '.$table_name.' ('.implode(', ', $fieldname).') VALUES ('.implode(', ', $inserts).')');

		return true;
	}

	public function insert_all($table_name, $fields, $data)
	{
		if(is_array($fields)) {
			$field_name = implode(', ', $fields);
		} else {
			$field_name = $fields;
		}
		if(!is_array($data)) {
			throw new Exception('data must array');
			return false;
		}
		if(!is_array($data[0])) {
			throw new Exception('data must 2dim array');
			return false;
		}

		while(count($data) > 100) {
			$sub = array_splice($data, 0, 100);
			foreach($sub as $rows) {
				foreach($rows as $value) {
					if(substr($value, 0, 1) == CHR00) {
						$inserts[] = substr($value, 1);
					} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
						$inserts[] = $value;
					} else {
						$inserts[] = '\''.$value.'\'';
					}
				}
				$block[] = '('.implode(', ', $inserts).')';
				unset($inserts);
			}
			$this->query('INSERT INTO '.$table_name.' ('.$field_name.') VALUES '.implode(', ', $block));
			unset($block);
		}
		foreach($data as $rows) {
			foreach($rows as $value) {
				if(substr($value, 0, 1) == CHR00) {
					$inserts[] = substr($value, 1);
				} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
					$inserts[] = $value;
				} else {
					$inserts[] = '\''.$value.'\'';
				}
			}
			$block[] = '('.implode(', ', $inserts).')';
		}
		$this->query('INSERT INTO '.$table_name.' ('.$field_name.') VALUES '.implode(', ', $block));
		
		return true;
	}

	public function update($table_name, $fields, $wheres = 1)
	{
		foreach($fields as $key => $value) {
			if(substr($value, 0, 1) == CHR00) {
			$updates[] = $key.'='.substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$updates[] = $key.'='.$value;
			} else {
				$updates[] = $key.'=\''.$value.'\'';
			}
		}
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == '' || $where == '()') {
			$where = 1;
		}

		$this->query('UPDATE '.$table_name.' SET '.implode(', ', $updates).' WHERE '.$where);

		return true;
	}

	public function replace($table_name, $fields)
	{
		foreach($fields as $key => $value) {
			$fieldname[] = $key;
			if(substr($value, 0, 1) == CHR00) {
				$replaces[] = substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$replaces[] = $value;
			} else {
				$replaces[] = '\''.$value.'\'';
			}
		}

		$this->query('REPLACE INTO '.$table_name.' ('.implode(', ', $fieldname).') VALUES ('.implode(', ', $replaces).')');

		return true;
	}

	public function delete($table_name, $wheres = '')
	{
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == '' || $where == '()') {
			return false;
		}

		$this->query('DELETE FROM '.$table_name.' WHERE '.$where);

		return true;
	}

	public function truncate($table_name)
	{
		$this->query('TRUNCATE TABLE '.$table_name);

		return true;
	}
}

?>
