<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +--------------------------------------------------------+
// | PHP version 5.0.x                                      |
// +--------------------------------------------------------+
// | Copyright (c) 2005 Song Hyo-Jin                        |
// +--------------------------------------------------------+
// | License : GPL (current new version)                    |
// +--------------------------------------------------------+
// | Author : Song Hyo-Jin <crosser at hanmail dot net>     |
// |                                  (MSN Messengerable)   |
// +--------------------------------------------------------+
//
// $Id: SHJORACLE.inc, v 0.1.1 2005/08/11 19:40:00 crosser Exp $
//
// ORACLE 8i database control

include_once 'SHJ/debug.inc';
include_once 'SHJ/DB/CommonSQL.inc';

class SHJORACLE
{
	public $result, $err_report, $query;
	private $transaction, $transaction_flag, $query_count, $select_count;
	protected $persistent, $conn, $db_name, $insert_table;

	public function __construct($db_ini_file, $persistent = false, $charset = false)
	{
		global $_DEBUG;
		$this->err_report = $_DEBUG;

		$this->transaction	= false;
		$this->transaction_flag = OCI_COMMIT_ON_SUCCESS;
		$this->query_count	= 0;
		$this->select_count = 0;
		$this->persistent = $persistent;
		static $db_set_file = array();

		if(false === array_key_exists($db_ini_file, $db_set_file)) {
			if(!($db_set_file[$db_ini_file] = parse_ini_file($db_ini_file))) {
				exit($db_ini_file.' not found');
			}
		}
		$myset = &$db_set_file[$db_ini_file];
		if(isset($myset['dsn'])) {
			if($this->persistent) {
				if($charset) {
					$this->conn = oci_pconnect($myset['user'], $myset['pass'], $myset['dsn'], $charset);
				} else {
					$this->conn = oci_pconnect($myset['user'], $myset['pass'], $myset['dsn']);
				}
			} else {
				if($charset) {
					$this->conn = oci_new_connect($myset['user'], $myset['pass'], $myset['dsn'], $charset);
				} else {
					$this->conn = oci_new_connect($myset['user'], $myset['pass'], $myset['dsn']);
				}
			}
			$this->db_name = '';
		} else {
			if($this->persistent) {
				if($charset) {
					$this->conn = oci_pconnect($myset['user'], $myset['pass'], '', $charset);
				} else {
					$this->conn = oci_pconnect($myset['user'], $myset['pass']);
				}
			} else {
				if($charset) {
					$this->conn = oci_new_connect($myset['user'], $myset['pass'], '', $charset);
				} else {
					$this->conn = oci_new_connect($myset['user'], $myset['pass']);
				}
			}
			$this->db_name = '';
		}

		return true;
	}

	public function begin()
	{
		if(!$this->transaction) {
			$this->transaction_flag = OCI_DEFAULT;
		} else {
			throw new Exception('already begined.');
			return false;
		}

		$this->transaction = true;

		return true;
	}

	public function commit()
	{
		if($this->transaction) {
			oci_commit($this->conn);
		} else {
			throw new Exception('not yet begin.');
			return false;
		}

		$this->transaction = false;
		$this->transaction_flag = OCI_COMMIT_ON_SUCCESS;

		return true;
	}

	public function rollback()
	{
		if($this->transaction) {
			oci_rollback($this->conn);
		} else {
			throw new Exception('not yet begin.');
			return false;
		}

		$this->transaction = false;
		$this->transaction_flag = OCI_COMMIT_ON_SUCCESS;

		return true;
	}

	public function __destruct()
	{
		if($this->conn !== false) {
			$this->close();
		}

		return true;
	}

	public function close()
	{
		if($this->conn === false) {
			return false;
		}
		if($this->transaction) {
			$this->rollback();
		}
		if($this->persistent === false) {
			oci_close($this->conn);
		}
		$this->conn = false;

		return true;
	}

	public function query($query)
	{
		$this->query = $query;
		$this->exec();

		return true;
	}

	public function exec()
	{
		$this->query_count ++;

		try {
			$this->result = oci_parse($this->conn, $this->query);
			oci_execute($this->result, $this->transaction_flag);
		} catch(Exception $e) {
			$this->result = false;
		}
		if(!$this->result) {
			if($this->err_report) {
				throw new Exception('<p>query <strong>'.htmlspecialchars($this->query).'</strong> error (count : '.$this->query_count.' )</p><xmp>'.$e->getMessage().'</xmp>');
			} else {
				throw new Exception('<p>query error (count : '.$this->query_count.' )</p>');
			}
		}
		return true;
	}

	public function id($seq = false)
	{
		if($seq) {
			$sql->query('SELECT '.$seq.'.CURRVAL FROM DUAL');
		} else {
			$sql->query('SELECT '.$this->insert_table.'_seq.CURRVAL FROM DUAL');
		}
		return $sql->field();
	}

	public function affected_rows()
	{
		return oci_num_rows($this->result);
	}

	public function field()
	{
		oci_fetch($this->result);
		return oci_result($this->result, 1);
	}

	public function fetch()
	{
		return oci_fetch_object($this->result);
	}

	public function fetch_row()
	{
		return oci_fetch_row($this->result);
	}

	public function fetch_assoc()
	{
		return oci_fetch_assoc($this->result);
	}

	public function fetch_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function fetch_row_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch_row())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function fetch_assoc_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch_assoc())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function select($tables, $fields, $wheres = '', $orders = '', $offset = 0, $limit = -1)
	{
		$this->select_count ++;

		$tables = (array)$tables;
		$srflag = false;
		foreach($tables as $value) {
			if(false !== strpos($value, ' ')) {
				$table[] = $value;
			} else {
				$table[] = $value.' SR'.$this->select_count;
				$srflag = true;
			}
		}
		$table_name = implode(', ', $table);

		$fields = (array)$fields;
		foreach($fields as $value) {
			if(!preg_match('/[\.\(]/', $value) && $srflag) {
				$field[] = 'SR'.$this->select_count.'.'.$value;
			} else {
				$field[] = $value;
			}
		}
		$field_name = implode(', ', $field);

		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == 1 or $where == '()') {
			$where = '';
		} elseif($where != '') {
			$where = ' WHERE '.$where;
		}
		if(is_array($orders)) {
			foreach($orders as $value) {
				if($value == 1 || $value == '') {
					continue;
				}
				if(false !== strpos($value, '.')) {
					$orders_tmp[] = $value;
				} else {
					$orders_tmp[] = 'SR'.$this->select_count.'.'.$value;
				}
			}
			if(isset($orders_tmp)) {
				$orders = $orders_tmp;
				unset($orders_tmp);
				$order = implode(', ', $orders);
			} else {
				$orders = array();
				$order = '';
			}
		} else {
			if($orders == 1 || $orders == '') {
				$order = '';
			} else {
				if(false !== strpos($orders, '.')) {
					$order = $orders;
				} else {
					$order = 'SR'.$this->select_count.'.'.$orders;
				}
			}
		}
		if($order != '') {
			$order		= ' ORDER BY '.$order;
		}

		if($offset != null) {
			if(!preg_match('/^\d+$/', $offset)) {
				throw new Exception('offset must numeric');
				return false;
			}
			if($offset != 0) {
				$rows[] = 'RNUM > '.$offset;
			}
		} else {
			$offset = 0;
		}
		if($limit != null) {
			if($limit != -1) {
				if(!preg_match('/^\d+$/', $limit)) {
					throw new Exception('limit must numeric');
					return false;
				} else {
					$rows[] = 'RNUM <= '.($offset + $limit);
				}
			}
		}

		if(isset($rows)) {
			$subquery = 'SELECT ROWNUM AS RNUM, '.$field_name.' FROM '.$table_name.$where.$order;
			$this->query = 'SELECT * FROM ('.$subquery.') WHERE '.implode(' AND ', $rows);
		} else {
			$this->query = 'SELECT '.$field_name.' FROM '.$table_name.$where.$order;
		}

		return true;
	}

	public function insert($table_name, $fields)
	{
		foreach($fields as $key => $value) {
			$fieldname[] = $key;
			if(substr($value, 0, 1) == CHR00) {
				$inserts[] = substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$inserts[] = $value;
			} else {
				$inserts[] = '\''.$value.'\'';
			}
		}

		$this->query('INSERT INTO '.$table_name.' ('.implode(', ', $fieldname).') VALUES ('.implode(', ', $inserts).')');
		$this->insert_table = $table_name;

		return true;
	}

	public function update($table_name, $fields, $wheres = 1)
	{
		foreach($fields as $key => $value) {
			if(substr($value, 0, 1) == CHR00) {
			$updates[] = $key.'='.substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$updates[] = $key.'='.$value;
			} else {
				$updates[] = $key.'=\''.$value.'\'';
			}
		}
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == '' || $where == '()') {
			$where = 1;
		}

		$this->query('UPDATE '.$table_name.' SET '.implode(', ', $updates).' WHERE '.$where);

		return true;
	}

	public function replace($table_name, $fields, $pkey = false)
	{
		if(!$pkey) {
			if(false === ($pkey = $this->get_pkey($table_name))) {
				throw new Exception('primary key not found');
				return false;
			}
		}
		if(!isset($fields[$pkey])) {
			throw new Exception('fields must have primary key');
			return false;
		}
		if(substr($fields[$pkey], 0, 1) == CHR00) {
			$where = $key.'='.substr($fields[$pkey], 1);
		} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
			$where = $key.'='.$fields[$pkey];
		} else {
			$where = $key.'=\''.$fields[$pkey].'\'';
		}

		$this->delete($table_name, $where);
		$this->insert($table_name, $fields);

		return true;
	}

	public function delete($table_name, $wheres = '')
	{
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == '' || $where == '()') {
			return false;
		}

		$this->query('DELETE FROM '.$table_name.' WHERE '.$where);

		return true;
	}

	public function truncate($table_name)
	{
		$this->query('TRUNCATE TABLE '.$table_name);

		return true;
	}

	private function get_pkey($pkey)
	{
		$this->query('SELECT A.COLUMN_NAME FROM USER_CONS_COLUMNS A, USER_CONSTRAINTS B WHERE A.TABLE_NAME = B.TABLE_NAME AND A.CONSTRAINT_NAME = B.CONSTRAINT_NAME AND B.CONSTRAINT_TYPE = \'P\' AND A.TABLE_NAME = \''.$pkey.'\'');
		if($this->rows() != 1) {
			return false;
		}
		return $this->field();
	}
}

?>
