<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +--------------------------------------------------------+
// | PHP version 5.0.x                                      |
// +--------------------------------------------------------+
// | Copyright (c) 2005 Song Hyo-Jin                        |
// +--------------------------------------------------------+
// | License : GPL (current new version)                    |
// +--------------------------------------------------------+
// | Author : Song Hyo-Jin <crosser at hanmail dot net>     |
// |                                  (MSN Messengerable)   |
// +--------------------------------------------------------+
//
// $Id: SHJPostgreSQL.inc, v 0.1.1 2005/08/11 19:40:00 crosser Exp $
//
// PostgreSQL 8.x database control

include_once 'SHJ/debug.inc';
include_once 'SHJ/DB/CommonSQL.inc';

class SHJPostgreSQL
{
	public $result, $err_report, $query;
	private $transaction, $query_count, $table_name;
	protected $persistent, $conn, $db_name;

	public function __construct($db_ini_file, $persistent = false, $charset = false)
	{
		global $_DEBUG;
		$this->err_report = $_DEBUG;

		$this->transaction	= false;
		$this->query_count	= 0;
		$this->persistent = $persistent;
		static $db_set_file = array();

		$this->table_name = false;

		if(false === array_key_exists($db_ini_file, $db_set_file)) {
			if(!($db_set_file[$db_ini_file] = parse_ini_file($db_ini_file))) {
				exit($db_ini_file.' not found');
			}
		}
		$myset = &$db_set_file[$db_ini_file];
		
		$this->db_name = $myset['db'];
		$connect = 'dbname='.$myset['db'].' user='.$myset['user'].' password='.$myset['pass'];
		if($myset['host'] != 'localhost') {
			$connect .= ' host='.$myset['ip'];
		}
		if($this->persistent) {
			$this->conn = pg_pconnect($connect, PGSQL_CONNECT_FORCE_NEW);
		} else {
			$this->conn = pg_connect($connect, PGSQL_CONNECT_FORCE_NEW);
		}

		if($charset) {
			pg_set_client_encoding($charset);
		}

		return true;
	}

	public function begin()
	{
		if(!$this->transaction) {
			pg_query($this->conn, 'BEGIN');
		} else {
			throw new Exception('already begined.');
			return false;
		}

		$this->transaction = true;

		return true;
	}

	public function commit()
	{
		if($this->transaction) {
			pg_query($this->conn, 'COMMIT');
		} else {
			throw new Exception('not yet begin.');
			return false;
		}

		$this->transaction = false;

		return true;
	}

	public function rollback()
	{
		if($this->transaction) {
			pg_query($this->conn, 'ROLLBACK');
		} else {
			throw new Exception('not yet begin.');
			return false;
		}

		$this->transaction = false;

		return true;
	}

	public function __destruct()
	{
		if($this->conn !== false) {
			$this->close();
		}

		return true;
	}

	public function close()
	{
		if($this->conn === false) {
			return false;
		}
		if($this->transaction) {
			$this->rollback();
		}
		if($this->persistent === false) {
			pg_close($this->conn);
		}

		$this->conn = false;

		return true;
	}

	public function query($query)
	{
		$this->query = $query;
		$this->exec();

		return true;
	}

	public function exec()
	{
		$this->query_count ++;

		try {
			$this->result = pg_query($this->conn, $this->query);
		} catch(Exception $e) {
			$this->result = false;
		}
		if(!$this->result) {
			if($this->err_report) {
				throw new Exception('<p>query <strong>'.htmlspecialchars($this->query).'</strong> error (count : '.$this->query_count.' )</p><xmp>'.pg_error($this->conn)."\n".$e->getMessage().'</xmp>');
			} else {
				throw new Exception('<p>query error (count : '.$this->query_count.' )</p>');
			}
		}
		return true;
	}

	public function rows()
	{
		return pg_num_rows($this->result);
	}

	public function id($seq = false)
	{
		if($seq === false) {
			if(false === $this->table_name) {
				return false;
			}
			$oid = $this->table_oid($this->table_name);
			if(false === ($seq = $this->get_seq($oid))) {
				return false;
			}
		}

		$this->query('SELECT currval(\''.$seq.'\')');
		return $this->field();
	}

	public function affected_rows()
	{
		return pg_affected_rows($this->conn);
	}

	public function field($row = 0, $col = 0)
	{
		return pg_fetch_result($this->result, $row, $col);
	}

	public function fetch()
	{
		return pg_fetch_object($this->result);
	}

	public function fetch_row()
	{
		return pg_fetch_row($this->result);
	}

	public function fetch_assoc()
	{
		return pg_fetch_assoc($this->result);
	}

	public function fetch_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function fetch_row_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch_row())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function fetch_assoc_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch_assoc())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function select($tables, $fields, $wheres = '', $orders = '', $offset = 0, $limit = -1)
	{
		if(is_array($tables)) {
			$table_name = implode(', ', $tables);
		} else {
			$table_name = $tables;
		}
		if(is_array($fields)) {
			$field_name = implode(', ', $fields);
		} else {
			$field_name = $fields;
		}
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == 1 or $where == '()') {
			$where = '';
		} elseif($where != '') {
			$where = ' WHERE '.$where;
		}
		if(is_array($orders)) {
			$order		= implode(', ', $orders);
		} else {
			$order		= $orders;
		}
		if($order == 1) {
			$order		= '';
		} elseif($order != '') {
			$order		= ' ORDER BY '.$order;
		}

		if($limit != null) {
			if($limit == -1) {
				$limit	= '';
			} elseif(!preg_match('/^\d+$/', $limit)) {
				throw new Exception('limit must numeric');
				return false;
			} else {
				$limit	= ' LIMIT '.$limit;
			}
		} else {
			$limit = '';
		}

		if($offset != null) {
			if(!preg_match('/^\d+$/', $offset)) {
				throw new Exception('offset must numeric');
				return false;
			}
			if($offset == 0) {
				$offset	= '';
			} else {
				$offset	= ' OFFSET '.$offset;
			}
		} else {
			$offset = '';
		}

		$this->query = 'SELECT '.$field_name.' FROM '.$table_name.$where.$order.$limit.$offset;

		return true;
	}

	public function insert($table_name, $fields)
	{
		$this->table_name = $table_name;

		foreach($fields as $key => $value) {
			$fieldname[] = $key;
			if(substr($value, 0, 1) == CHR00) {
				$inserts[] = substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$inserts[] = $value;
			} else {
				$inserts[] = '\''.$value.'\'';
			}
		}

		$this->query('INSERT INTO '.$table_name.' ('.implode(', ', $fieldname).') VALUES ('.implode(', ', $inserts).')');

		return true;
	}

	public function insert_all($table_name, $fields, $data)
	{
		$this->table_name = $table_name;

		if(is_array($fields)) {
			$field_name = implode(', ', $fields);
		} else {
			$field_name = $fields;
		}
		if(!is_array($data)) {
			throw new Exception('data must array');
			return false;
		}
		if(!is_array($data[0])) {
			throw new Exception('data must 2dim array');
			return false;
		}
		
		while(count($data) > 100) {
			$sub = array_splice($data, 0, 100);
			pg_query($this->conn, 'COPY '.$table_name.' ('.$field_name.') FROM stdin');
			foreach($sub as $rows) {
				pg_put_line($this->conn, implode("\t", $rows)."\n");
			}
			pg_put_line($this->conn, "\\.\n");
			pg_end_copy($this->conn);
		}
		pg_query($this->conn, 'COPY '.$table_name.' ('.$field_name.') FROM stdin');
		foreach($data as $rows) {
			pg_put_line($this->conn, implode("\t", $rows)."\n");
		}
		pg_end_copy($this->conn);
		
		return true;
	}

	public function update($table_name, $fields, $wheres = 1)
	{
		foreach($fields as $key => $value) {
			if(substr($value, 0, 1) == CHR00) {
			$updates[] = $key.'='.substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$updates[] = $key.'='.$value;
			} else {
				$updates[] = $key.'=\''.$value.'\'';
			}
		}
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == '' || $where == '()') {
			$where = 1;
		}

		$this->query('UPDATE '.$table_name.' SET '.implode(', ', $updates).' WHERE '.$where);

		return true;
	}

	public function replace($table_name, $fields, $pkey = false)
	{
		if(!$pkey) {
			$oid = $this->table_oid($table_name);
			if(false === ($pkey = $this->get_pkey($oid))) {
				throw new Exception('primary key not found');
				return false;
			}
		}
		if(!isset($fields[$pkey])) {
			throw new Exception('fields must have primary key');
			return false;
		}
		if(substr($fields[$pkey], 0, 1) == CHR00) {
			$where = $key.'='.substr($fields[$pkey], 1);
		} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
			$where = $key.'='.$fields[$pkey];
		} else {
			$where = $key.'=\''.$fields[$pkey].'\'';
		}

		$this->delete($table_name, $where);
		$this->insert($table_name, $fields);

		return true;
	}

	public function delete($table_name, $wheres = '')
	{
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == '' || $where == '()') {
			return false;
		}

		$this->query('DELETE FROM '.$table_name.' WHERE '.$where);

		return true;
	}

	public function truncate($table_name)
	{
		$this->query('TRUNCATE TABLE '.$table_name);

		return true;
	}

	private function table_oid($table_name)
	{
		if(!preg_match('/^(.+)\.(.+)$/', $table_name, $matches)) {
			$matches[1] = 'public';
			$matches[2] = $table_name;
		}
		$this->query('SELECT c.oid FROM pg_catalog.pg_class c LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace WHERE n.nspname = \''.$matches[1].'\' AND c.relname = \''.$matches[2].'\' ORDER BY n.nspname, c.relname');
		if($row == 0) {
			throw new Exception('table ['.$table_name.'] not found');
			return false;
		}
		return $this->field($row - 1);
	}

	private function get_pkey($oid)
	{
		$this->query('SELECT a.attname FROM pg_catalog.pg_attribute a, pg_catalog.pg_index i WHERE a.attrelid = '.$oid.' AND a.attnum > 0 AND NOT a.attisdropped AND a.attrelid = i.indrelid AND i.indisprimary = TRUE LIMIT 1');
		if($this->rows() != 1) {
			return false;
		}
		return $this->field();
	}
	
	private function get_seq($oid)
	{
		$this->query('SELECT d.adsrc FROM pg_catalog.pg_attribute a, pg_catalog.pg_attrdef d WHERE a.attrelid = '.$oid.' AND a.attnum > 0 AND NOT a.attisdropped AND d.adrelid = a.attrelid AND d.adnum = a.attnum AND a.atthasdef AND d.adsrc ~ \'^nextval\'');
		if($this->rows() != 1) {
			return false;
		}
		$temp = $this->field();
		if(!preg_match('/\'(.+)\'/U', $temp, $matches)) {
			return false;
		}
		return $matches[1];
	}
}

?>
