<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +--------------------------------------------------------+
// | PHP version 5.0.x                                      |
// +--------------------------------------------------------+
// | Copyright (c) 2005 Song Hyo-Jin                        |
// +--------------------------------------------------------+
// | License : GPL (current new version)                    |
// +--------------------------------------------------------+
// | Author : Song Hyo-Jin <crosser at hanmail dot net>     |
// |                                  (MSN Messengerable)   |
// +--------------------------------------------------------+
//
// $Id: SHJSQLite.inc, v 0.1.1 2005/08/11 19:40:00 crosser Exp $
//
// SQLite 2.8.x database control

include_once 'SHJ/debug.inc';
include_once 'SHJ/DB/CommonSQL.inc';

class SHJSQLite
{
	public $err_report, $query;
	private $result, $transaction, $query_count;
	protected $class, $db_name;

	public function __construct($db_file)
	{
		global $_DEBUG;
		$this->err_report = $_DEBUG;

		$this->transaction	= false;
		$this->query_count	= 0;

		$this->class = new SQLiteDatabase($db_file);

		return true;
	}

	public function begin()
	{
		if(!isset($this->transaction)) {
			$this->transaction = false;
		}
		if(!$this->transaction) {
			$this->class->query('BEGIN');
		} else {
			throw new Exception('already begined.');
			return false;
		}

		$this->transaction = true;

		return true;
	}

	public function commit()
	{
		if(!isset($this->transaction)) {
			$this->transaction = false;
		}
		if($this->transaction) {
			$this->class->query('COMMIT');
		} else {
			throw new Exception('not yet begin.');
			return false;
		}

		$this->transaction = false;

		return true;
	}

	public function rollback()
	{
		if(!isset($this->transaction)) {
			$this->transaction = false;
		}
		if($this->transaction) {
			$this->class->query('ROLLBACK');
		} else {
			throw new Exception('not yet begin.');
			return false;
		}

		$this->transaction = false;

		return true;
	}

	public function __destruct()
	{
		if($this->class !== false) {
			$this->close();
		}

		return true;
	}

	public function close()
	{
		if($this->class === false) {
			return false;
		}
		if($this->transaction) {
			$this->rollback();
		}

		$this->class = false;

		return true;
	}

	public function query($query)
	{
		$this->query = $query;
		$this->exec();

		return true;
	}

	public function exec()
	{
		$this->query_count ++;

		try {
			$this->result = $this->class->query($this->query);
		} catch(Exception $e) {
			$this->result = false;
		}
		if(!$this->result) {
			if($this->err_report) {
				throw new Exception('<p>query <strong>'.htmlspecialchars($this->query).'</strong> error (count : '.$this->query_count.' )</p><xmp>'.sqlite_error_string($this->class->lastError())."\n".$e->getMessage().'</xmp>');
			} else {
				throw new Exception('<p>query error (count : '.$this->query_count.' )</p>');
			}
		}
		return true;
	}

	public function rows()
	{
		return $this->result->numRows();
	}

	public function id()
	{
		return $this->class->lastInsertRowid();
	}

	public function affected_rows()
	{
		return $this->class->changes();
	}

	public function field($row = 0, $col = 0)
	{
		if(false === $this->result->seek($row)) {
			return false;
		}
		return $this->result->column($col);
	}

	public function fetch()
	{
		if(false === ($temp = $this->result->fetch(SQLITE_ASSOC))) {
			return false;
		}
		foreach($temp as $key => $value) {
			$key = preg_replace('/^[^\.]+\./', '', $key);
			$ret->$key = $value;
		}
		return $ret;
	}

	public function fetch_row()
	{
		return $this->result->fetch(SQLITE_NUM);
	}

	public function fetch_assoc()
	{
		if(false === ($temp = $this->result->fetch(SQLITE_ASSOC))) {
			return false;
		}
		foreach($temp as $key => $value) {
			$key = preg_replace('/^[^\.]+\./', '', $key);
			$ret[$key] = $value;
		}
		return $ret;
	}

	public function fetch_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function fetch_row_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch_row())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function fetch_assoc_all()
	{
		$ret = array();
		while(false !== ($temp = $this->fetch_assoc())) {
			$ret[] = $temp;
		}

		if(count($ret) == 0) {
			return false;
		}

		return $ret;
	}

	public function select($tables, $fields, $wheres = '', $orders = '', $offset = 0, $limit = -1)
	{
		if(is_array($tables)) {
			$table_name = implode(', ', $tables);
		} else {
			$table_name = $tables;
		}
		if(is_array($fields)) {
			$field_name = implode(', ', $fields);
		} else {
			$field_name = $fields;
		}
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == 1 or $where == '()') {
			$where = '';
		} elseif($where != '') {
			$where = ' WHERE '.$where;
		}
		if(is_array($orders)) {
			$order		= implode(', ', $orders);
		} else {
			$order		= $orders;
		}
		if($order == 1) {
			$order		= '';
		} elseif($order != '') {
			$order		= ' ORDER BY '.$order;
		}

		if($limit != null) {
			if($limit == -1) {
				$limit	= '';
			} elseif(!preg_match('/^\d+$/', $limit)) {
				throw new Exception('limit must numeric');
				return false;
			} else {
				$limit	= ' LIMIT '.$limit;
			}
		} else {
			$limit = '';
		}

		if($offset != null) {
			if(!preg_match('/^\d+$/', $offset)) {
				throw new Exception('offset must numeric');
				return false;
			}
			if($offset == 0) {
				$offset	= '';
			} else {
				$offset	= ' OFFSET '.$offset;
			}
		} else {
			$offset = '';
		}

		$this->query = 'SELECT '.$field_name.' FROM '.$table_name.$where.$order.$limit.$offset;

		return true;
	}

	public function insert($table_name, $fields)
	{
		foreach($fields as $key => $value) {
			$fieldname[] = $key;
			if(substr($value, 0, 1) == CHR00) {
				$inserts[] = substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$inserts[] = $value;
			} else {
				$inserts[] = '\''.$value.'\'';
			}
		}

		$this->query('INSERT INTO '.$table_name.' ('.implode(', ', $fieldname).') VALUES ('.implode(', ', $inserts).')');

		return true;
	}

	public function update($table_name, $fields, $wheres = 1)
	{
		foreach($fields as $key => $value) {
			if(substr($value, 0, 1) == CHR00) {
			$updates[] = $key.'='.substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$updates[] = $key.'='.$value;
			} else {
				$updates[] = $key.'=\''.$value.'\'';
			}
		}
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == '' || $where == '()') {
			$where = 1;
		}

		$this->query('UPDATE '.$table_name.' SET '.implode(', ', $updates).' WHERE '.$where);

		return true;
	}

	public function replace($table_name, $fields)
	{
		foreach($fields as $key => $value) {
			$fieldname[] = $key;
			if(substr($value, 0, 1) == CHR00) {
				$replaces[] = substr($value, 1);
			} elseif(preg_match('/^([1-9]\d*|0)$/', $value)) {
				$replaces[] = $value;
			} else {
				$replaces[] = '\''.$value.'\'';
			}
		}

		$this->query('REPLACE INTO '.$table_name.' ('.implode(', ', $fieldname).') VALUES ('.implode(', ', $replaces).')');

		return true;
	}

	public function delete($table_name, $wheres = '')
	{
		if(is_array($wheres)) {
			$where		= '('.implode(') AND (', $wheres).')';
		} else {
			$where		= $wheres;
		}
		if($where == '' || $where == '()') {
			return false;
		}

		$this->query('DELETE FROM '.$table_name.' WHERE '.$where);

		return true;
	}

	public function truncate($table_name)
	{
		$this->query('TRUNCATE TABLE '.$table_name);

		return true;
	}
}

?>
